@@ -20,7 +20,7 @@
 #include <math.h>
 #include "crinkle.h"
 
-char calcalt_Id[] = "$Id: calcalt.c,v 2.6 1995/06/12 18:58:04 spb Exp spb $";
+char calcalt_Id[] = "$Id: calcalt.c,v 2.12 1996/09/13 10:07:46 spb Exp spb $";
 
 #ifdef DEBUG
 #define DB(A,B) dump_pipeline(A,B)
@@ -29,12 +29,12 @@ char calcalt_Id[] = "$Id: calcalt.c,v 2.6 1995/06/12 18:58:04 spb Exp spb $";
 #endif
 
 
-/*{{{  Strip *make_strip(int level) */
-Strip *make_strip (level)
-int level;
+/*{{{  Strip *make_strip(Fold *f) */
+Strip *make_strip (f)
+Fold *f;
 {
   Strip *p;
-  int  points;
+  int n;
 
   p = (Strip *) malloc( sizeof(Strip) );
   if( p == NULL )
@@ -42,9 +42,13 @@ int level;
     fprintf(stderr,"make_strip: malloc failed\n");
     exit(1);
   }
-  p->level = level;
-  points = (1 << level) +1;
-  p->d = (Height *)malloc( points * sizeof(Height) );
+  p->f = f;
+#ifdef MPI
+  n = f->size;
+#else
+  n = f->count;
+#endif
+  p->d = (Height *)malloc( n * sizeof(Height) );
   if( p->d == NULL )
   {
     fprintf(stderr,"make_strip: malloc failed\n");
@@ -72,11 +76,16 @@ Strip *s;
   Strip *p;
   Height *a, *b;
   int i;
+  int count;
 
-  p = make_strip((s->level)+1);
+  p = make_strip(s->f->parent);
+#ifdef MPI
+  redistribude data
+#else
   a = s->d;
   b = p->d;
-  for(i=0; i < (1<<s->level); i++)
+  count = s->f->count;
+  for(i=0; i < count-1; i++)
   {
     *b = *a;
     a++;
@@ -85,21 +94,28 @@ Strip *s;
     b++;
   }
   *b = *a;
+#endif
   return(p);
 }
 /*}}}*/
-/*{{{  Strip *set_strip(int level, Height value) */
-Strip *set_strip (level,value)
-int level;
+/*{{{  Strip *set_strip(Fold *f, Height value)*/
+Strip *set_strip (f,value)
+Fold *f;
 Height value;
 {
   int i;
   Strip *s;
   Height *h;
-
-  s = make_strip(level);
+  int count;
+  
+  s = make_strip(f);
   h = s->d;
-  for( i=0 ; i < ((1<<level)+1) ; i++)
+#ifdef MPI
+  count = f->size;
+#else
+  count = f->count;
+#endif
+  for( i=0 ; i < count ; i++)
   {
     *h = value;
     h++;
@@ -107,7 +123,28 @@ Height value;
   return(s);
 }
 /*}}}*/
-/*{{{  Fold *make_fold(Parm *param, int levels, int stop, Length len) */
+/*{{{  Strip *random_strip(Fold *f)*/
+Strip *random_strip(f)
+Fold *f;
+{
+  Strip *result;
+  int i, count;
+  
+  result=make_strip(f);
+#ifdef MPI
+  count=f->size;  /* this is a little wasteful as it also sets the halo */
+#else
+  count = f->count;
+#endif
+  for( i=0 ; i < count ; i++)
+  {
+    result->d[i] = f->p->mean + (f->scale * gaussian());
+  }
+  return(result);
+}
+/*}}}*/
+
+/*{{{  Fold *make_fold(Fold *parent,Parm *param, int levels, int stop, Length len) */
 /*
  * Initialise the fold structures.
  * As everything else reads the parameters from their fold
@@ -124,7 +161,8 @@ Height value;
  *   len gets smaller as the level increases.
  * start, the starting height for a non-fractal start.
  */
-Fold *make_fold (param,levels,stop,length)
+Fold *make_fold (parent,param,levels,stop,length)
+struct fold *parent;
 struct parm *param;
 int levels;
 int stop;
@@ -151,6 +189,7 @@ Length length;
   scale = pow((double) length, (double) (2.0 * param->fdim));
   midscale = pow((((double) length)*root2), (double) (2.0 * param->fdim));
   p->level = levels;
+  p->count = (1 << levels) +1;
   p->stop = stop;
   p->state = START;
   p->save =NULL;
@@ -161,9 +200,14 @@ Length length;
   {
     p->s[i] = NULL;
   }
+  p->parent=parent;
+#ifdef MPI
+  /* work out the distribution at this level */
+  p->first = ( p->p-me == 0 );
+#endif
   if( levels > stop )
   {
-    p->next = make_fold(param,(levels-1),stop,(2.0*length));
+    p->next = make_fold(p,param,(levels-1),stop,(2.0*length));
   }else{
     p->next = NULL;
   }
@@ -195,17 +239,13 @@ Fold *fold;
   Strip *result=NULL;
   Strip *tmp;
   Strip **t;
-  int i, count, iter;
+  int i, iter;
+  int count=fold->count;
 
-  count = ((1 << fold->level) +1);
   if( fold->level == fold->stop)
   {
     /*{{{  generate values from scratch */
-    result=make_strip(fold->stop);
-    for( i=0 ; i < count ; i++)
-    {
-      result->d[i] = fold->p->mean + (fold->scale * gaussian());
-    }
+    result=random_strip(fold);
     /*}}}*/
   }else{
   /*
@@ -232,7 +272,7 @@ Fold *fold;
           t[0] = double_strip(tmp);
           free_strip(tmp);
           /* make the new B strip */
-          t[1]=set_strip(fold->level,0.0);
+          t[1]=set_strip(fold,0.0);
           if( ! t[2] )
           {
             /* we want to have an A B A pattern of strips at the
@@ -251,7 +291,7 @@ Fold *fold;
            * t := A B A
            */
           DB("S2",fold);
-          x_update(count,fold->midscale,0.0,t[0],t[1],t[2]);
+          x_update(fold,fold->midscale,0.0,t[0],t[1],t[2]);
           DB("E2",fold);
           
           if(fold->p->rg1)
@@ -262,7 +302,13 @@ Fold *fold;
              * use the midpoints to regenerate the corner values
              * increment t by 2 so we still have and A B A pattern
              */
-            v_update(count,fold->midscale,fold->p->midmix,t[1],t[2],t[3]);
+            if( t[3] == NULL )
+            {
+              /* rather than do no update add offset to old value */
+              v_update(fold,fold->midscale,1.0,t[1],t[2],t[1]);
+            }else{
+              v_update(fold,fold->midscale,fold->p->midmix,t[1],t[2],t[3]);
+            }
             t+=2;
             DB("E3",fold);
           
@@ -276,12 +322,12 @@ Fold *fold;
           
           if( fold->p->cross )
           {
-            t_update(count,fold->scale,0.0,t[0],t[1],t[2]);
-            p_update(count,fold->scale,0.0,t[1],t[2],t[3]);
+            t_update(fold,fold->scale,0.0,t[0],t[1],t[2]);
+            p_update(fold,fold->scale,0.0,t[1],t[2],t[3]);
             t+=2;
           }else{
-            hside_update(count,fold->scale,0.0,t[0],t[1],t[2]);
-            vside_update(count,fold->scale,0.0,t[2]);
+            hside_update(fold,fold->scale,0.0,t[0],t[1],t[2]);
+            vside_update(fold,fold->scale,0.0,t[2]);
             t+=2;
           }
           DB("E4",fold);
@@ -295,9 +341,15 @@ Fold *fold;
              */
             if( fold->p->cross )
             {
-              p_update(count,fold->scale,fold->p->mix,t[0],t[1],t[2]);
+              if( t[2] == NULL )
+              {
+                /* add random offset to old rather than skip update */
+                p_update(fold,fold->scale,fold->p->mix,t[0],t[1],t[0]);
+              }else{
+                p_update(fold,fold->scale,fold->p->mix,t[0],t[1],t[2]);
+              }
             }else{
-              vside_update(count,fold->scale,fold->p->mix,t[1]);
+              vside_update(fold,fold->scale,fold->p->mix,t[1]);
             }
             DB("E5",fold);
           
@@ -319,7 +371,13 @@ Fold *fold;
              *
              * this has to be a t_update
              */
-            t_update(count,fold->scale,fold->p->mix,t[0],t[1],t[2]);
+            if( t[2] == NULL )
+            {
+              /* add random offset to old rather than skip update */
+              t_update(fold,fold->scale,1.0,t[0],t[1],t[0]);
+            }else{
+              t_update(fold,fold->scale,fold->p->mix,t[0],t[1],t[2]);
+            }
             t++;
             DB("E6",fold);
           
@@ -361,9 +419,9 @@ Fold *fold;
 }
 /*}}}*/
 
-/*{{{void x_update(int count,float scale, float mix, Strip *a, Strip *b, Strip *c)*/
-void x_update(count, scale, mix, a, b, c)
-int count;
+/*{{{void x_update(Fold *fold,float scale, float mix, Strip *a, Strip *b, Strip *c)*/
+void x_update(fold, scale, mix, a, b, c)
+Fold *fold;
 float scale;
 float mix;
 Strip *a;
@@ -371,6 +429,7 @@ Strip *b;
 Strip *c;
 {
   int i;
+  int count=fold->count;
   float w;
   Height *mp, *lp, *rp;
 
@@ -382,7 +441,7 @@ Strip *c;
     exit(1);
   }
   
-  w = (1.0 - mix)/4.0;
+  w = (1.0 - mix)*0.25;
   mp=b->d;
   lp=a->d;
   rp=c->d;
@@ -423,9 +482,9 @@ Strip *c;
   }
 }    
 /*}}}*/
-/*{{{void p_update(int count,float scale, float mix, Strip *a, Strip *b, Strip *c)*/
-void p_update(count, scale, mix, a, b, c)
-int count;
+/*{{{void p_update(Fold *fold,float scale, float mix, Strip *a, Strip *b, Strip *c)*/
+void p_update(fold, scale, mix, a, b, c)
+Fold *fold;
 float scale;
 float mix;
 Strip *a;
@@ -433,6 +492,7 @@ Strip *b;
 Strip *c;
 {
   int i;
+  int count=fold->count;
   float w;
   Height *mp, *lp, *rp;
 
@@ -445,11 +505,11 @@ Strip *c;
    */
   if( !c )
   {
-    vside_update(count,scale,mix,b);
+    vside_update(fold,scale,mix,b);
     return;
   }
 
-  w = (1.0 - mix)/4.0;
+  w = (1.0 - mix)*0.25;
   mp=b->d;
   lp=a->d;
   rp=c->d;
@@ -491,9 +551,9 @@ Strip *c;
 }    
 
 /*}}}*/
-/*{{{void t_update(int count,float scale, float mix, Strip *a, Strip *b, Strip *c)*/
-void t_update(count, scale, mix, a, b, c)
-int count;
+/*{{{void t_update(Fold *fold,float scale, float mix, Strip *a, Strip *b, Strip *c)*/
+void t_update(fold, scale, mix, a, b, c)
+Fold *fold;
 float scale;
 float mix;
 Strip *a;
@@ -501,8 +561,10 @@ Strip *b;
 Strip *c;
 {
   int i;
+  int count=fold->count;
   float w, we;
   Height *mp, *lp, *rp;
+  float third=(1.0/3.0);
 
   /* don't run unless we have all the parameters */
   if( !a || !c ) return;
@@ -512,15 +574,15 @@ Strip *c;
     exit(1);
   }
 
-  w = (1.0 - mix)/4.0;
-  we = (1.0 - mix)/3.0;
+  w = (1.0 - mix)*0.25;
+  we = (1.0 - mix)*third;
   mp=b->d;
   lp=a->d;
   rp=c->d;
 
   if( mix <= 0.0){
     /*{{{random offset to average of new points*/
-    mp[0] = (1.0/3.0) * ( lp[0] + rp[0] + mp[1] )
+    mp[0] = third * ( lp[0] + rp[0] + mp[1] )
             + (scale * gaussian());
     mp++;
     lp++;
@@ -533,12 +595,12 @@ Strip *c;
       lp+=2;
       rp+=2;
     }
-    mp[1] = (1.0/3.0) * ( lp[1] + rp[1] + mp[0] )
+    mp[1] = third * ( lp[1] + rp[1] + mp[0] )
           + (scale * gaussian());
     /*}}}*/
   }else if(mix >= 1.0){
     /*{{{random offset to old values*/
-    for(i=0; i<count-2; i+=2)
+    for(i=0; i<count; i+=2)
     {
       mp[0] = mp[0]
             + (scale * gaussian());
@@ -570,9 +632,9 @@ Strip *c;
 
 
 /*}}}*/
-/*{{{void v_update(int count,float scale, float mix, Strip *a, Strip *b, Strip *c)*/
-void v_update(count, scale, mix, a, b, c)
-int count;
+/*{{{void v_update(Fold *fold,float scale, float mix, Strip *a, Strip *b, Strip *c)*/
+void v_update(fold, scale, mix, a, b, c)
+Fold *fold;
 float scale;
 float mix;
 Strip *a;
@@ -580,6 +642,7 @@ Strip *b;
 Strip *c;
 {
   int i;
+  int count=fold->count;
   float w, we;
   Height *mp, *lp, *rp;
 
@@ -591,8 +654,8 @@ Strip *c;
     exit(1);
   }
 
-  w = (1.0 - mix)/4.0;
-  we = (1.0 - mix)/2.0;
+  w = (1.0 - mix)*0.25;
+  we = (1.0 - mix)*0.5;
   mp=b->d;
   lp=a->d;
   rp=c->d;
@@ -617,7 +680,7 @@ Strip *c;
     /*}}}*/
   }else if(mix >= 1.0){
     /*{{{random offset to old values*/
-    for(i=0; i<count-2; i+=2)
+    for(i=0; i<count; i+=2)
     {
       mp[0] = mp[0]
             + (scale * gaussian());
@@ -648,14 +711,15 @@ Strip *c;
 }    
 
 /*}}}*/
-/*{{{void vside_update(int count,float scale, float mix, Strip *a)*/
-void vside_update(count, scale, mix, a)
-int count;
+/*{{{void vside_update(Fold *fold,float scale, float mix, Strip *a)*/
+void vside_update(fold, scale, mix, a)
+Fold *fold;
 float scale;
 float mix;
 Strip *a;
 {
   int i;
+  int count=fold->count;
   float w;
   Height *mp;
 
@@ -663,7 +727,7 @@ Strip *a;
   if( !a ) return;
 
 
-  w = (1.0 - mix)/2.0;
+  w = (1.0 - mix)*0.5;
   mp=a->d;
 
   if( mix <= 0.0){
@@ -698,9 +762,9 @@ Strip *a;
 
 
 /*}}}*/
-/*{{{void hside_update(int count,float scale, float mix, Strip *a, Strip *b, Strip *c)*/
-void hside_update(count, scale, mix, a, b, c)
-int count;
+/*{{{void hside_update(Fold *fold,float scale, float mix, Strip *a, Strip *b, Strip *c)*/
+void hside_update(fold, scale, mix, a, b, c)
+Fold *fold;
 float scale;
 float mix;
 Strip *a;
@@ -708,6 +772,7 @@ Strip *b;
 Strip *c;
 {
   int i;
+  int count=fold->count;
   float w;
   Height *mp, *lp, *rp;
 
@@ -719,7 +784,7 @@ Strip *c;
     exit(1);
   }
 
-  w = (1.0 - mix)/2.0;
+  w = (1.0 - mix)*0.5;
   mp=b->d;
   lp=a->d;
   rp=c->d;
